use std::env;
use std::io::{prelude::*, BufReader};
use std::path::Path;
use std::fs::{File};
use csv::WriterBuilder;
use std::error::Error;


enum Sentiment {
    Negative,
    Positive
}

struct Datapoint {
    text: String,
    sentiment: Sentiment
}

fn main() {
    let args: Vec<String> = env::args().collect();

    let pos_source = Path::new(&args[1]);
    let neg_source = Path::new(&args[2]);
    let target = Path::new(&args[3]);

    let data = parse_files(pos_source, neg_source);

    write_data(target, &data).expect("Couldn't write CSV")
}


fn parse_files(pos_source: &Path, neg_source: &Path) -> Vec<Datapoint>{
    let pos_file = File::open(pos_source).expect("Couldn't open file");
    let neg_file = File::open(neg_source).expect("Couldn't open file");

    let mut result: Vec<Datapoint> = Vec::new();

    for line in BufReader::new(pos_file).lines() {

        result.push(Datapoint { text: line.expect("Couldn't read line"), sentiment: Sentiment::Positive })
    }

    for line in BufReader::new(neg_file).lines() {

        result.push(Datapoint { text: line.expect("Couldn't read line"), sentiment: Sentiment::Negative })
    }

    result
}

fn write_data(file: &Path, data: &Vec<Datapoint>)-> Result<(), Box<dyn Error>>{
    let mut writer = WriterBuilder::new().delimiter(b';').has_headers(true).quote_style(csv::QuoteStyle::NonNumeric).from_path(file)?;
    writer.write_record(&["text", "sentiment"])?;
    for point in data {
        writer.serialize((&point.text, match &point.sentiment { Sentiment::Positive => 1,    Sentiment::Negative => 0}))?;
    }

    writer.flush()?;
    Ok(())
}
